@@ -2,26 +2,25 @@ import logging
 import re
 import struct
 from concurrent.futures import Future
+from datetime import datetime, timedelta
 from typing import Any, Dict, List, NamedTuple, Optional, Tuple, Union
 from urllib.parse import urlparse
 
-# noinspection PyPackageRequirements
-from Crypto.Cipher import AES
-
-# noinspection PyPackageRequirements
-from Crypto.Util.Padding import unpad
 from requests import Response
-from requests.exceptions import ChunkedEncodingError, ConnectionError, ContentDecodingError
+from requests.exceptions import ChunkedEncodingError, ConnectionError, ContentDecodingError, InvalidSchema
 
 from streamlink.buffers import RingBuffer
 from streamlink.exceptions import StreamError
+from streamlink.session import Streamlink
 from streamlink.stream.ffmpegmux import FFMPEGMuxer, MuxedStream
 from streamlink.stream.filtered import FilteredStream
 from streamlink.stream.hls_playlist import M3U8, ByteRange, Key, Map, Media, Segment, load as load_hls_playlist
 from streamlink.stream.http import HTTPStream
 from streamlink.stream.segmented import SegmentedStreamReader, SegmentedStreamWorker, SegmentedStreamWriter
 from streamlink.utils.cache import LRUCache
+from streamlink.utils.crypto import AES, unpad
 from streamlink.utils.formatter import Formatter
+from streamlink.utils.times import now
 
 
 log = logging.getLogger(__name__)
@@ -112,12 +111,20 @@ class HLSStreamWriter(SegmentedStreamWriter):
             key_uri = formatter.format(self.key_uri_override)
 
         if key_uri and self.key_uri != key_uri:
-            res = self.session.http.get(
-                key_uri,
-                exception=StreamError,
-                retries=self.retries,
-                **self.reader.request_params,
-            )
+            try:
+                res = self.session.http.get(
+                    key_uri,
+                    exception=StreamError,
+                    retries=self.retries,
+                    **self.reader.request_params,
+                )
+            except StreamError as err:
+                # FIXME: fix HTTPSession.request()
+                original_error = getattr(err, "err", None)
+                if isinstance(original_error, InvalidSchema):
+                    raise StreamError(f"Unable to find connection adapter for key URI: {key_uri}") from original_error
+                raise  # pragma: no cover
+
             res.encoding = "binary/octet-stream"
             self.key_data = res.content
             self.key_uri = key_uri
@@ -144,7 +151,7 @@ class HLSStreamWriter(SegmentedStreamWriter):
 
         return request_params
 
-    def put(self, sequence: Sequence):
+    def put(self, sequence: Optional[Sequence]):
         if self.closed:
             return
 
@@ -282,16 +289,22 @@ class HLSStreamWorker(SegmentedStreamWorker):
     writer: "HLSStreamWriter"
     stream: "HLSStream"
 
+    SEGMENT_QUEUE_TIMING_THRESHOLD_MIN = 5.0
+
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
         self.playlist_changed = False
         self.playlist_end: Optional[int] = None
+        self.playlist_targetduration: float = 0
         self.playlist_sequence: int = -1
         self.playlist_sequences: List[Sequence] = []
+        self.playlist_sequences_last: datetime = now()
+        self.playlist_reload_last: datetime = now()
         self.playlist_reload_time: float = 6
         self.playlist_reload_time_override = self.session.options.get("hls-playlist-reload-time")
         self.playlist_reload_retries = self.session.options.get("hls-playlist-reload-attempts")
+        self.segment_queue_timing_threshold_factor = self.session.options.get("hls-segment-queue-threshold")
         self.live_edge = self.session.options.get("hls-live-edge")
         self.duration_offset_start = int(self.stream.start_offset + (self.session.options.get("hls-start-offset") or 0))
         self.duration_limit = self.stream.duration or (
@@ -330,7 +343,7 @@ class HLSStreamWorker(SegmentedStreamWorker):
         try:
             playlist = self._reload_playlist(res)
         except ValueError as err:
-            raise StreamError(err)
+            raise StreamError(err) from err
 
         if playlist.is_master:
             raise StreamError(f"Attempted to play a variant playlist, use 'hls://{self.stream.url}' instead")
@@ -342,6 +355,7 @@ class HLSStreamWorker(SegmentedStreamWorker):
         sequences = [Sequence(media_sequence + i, s)
                      for i, s in enumerate(playlist.segments)]
 
+        self.playlist_targetduration = playlist.targetduration or 0
         self.playlist_reload_time = self._playlist_reload_time(playlist, sequences)
 
         if sequences:
@@ -354,8 +368,8 @@ class HLSStreamWorker(SegmentedStreamWorker):
             return sum(s.segment.duration for s in sequences[-max(1, self.live_edge - 1):])
         if type(self.playlist_reload_time_override) is float and self.playlist_reload_time_override > 0:
             return self.playlist_reload_time_override
-        if playlist.target_duration:
-            return playlist.target_duration
+        if playlist.targetduration:
+            return playlist.targetduration
         if sequences:
             return sum(s.segment.duration for s in sequences[-max(1, self.live_edge - 1):])
 
@@ -387,6 +401,20 @@ class HLSStreamWorker(SegmentedStreamWorker):
     def valid_sequence(self, sequence: Sequence) -> bool:
         return sequence.num >= self.playlist_sequence
 
+    def _segment_queue_timing_threshold_reached(self) -> bool:
+        if self.segment_queue_timing_threshold_factor <= 0:
+            return False
+
+        threshold = max(
+            self.SEGMENT_QUEUE_TIMING_THRESHOLD_MIN,
+            self.playlist_targetduration * self.segment_queue_timing_threshold_factor,
+        )
+        if now() <= self.playlist_sequences_last + timedelta(seconds=threshold):
+            return False
+
+        log.warning(f"No new segments in playlist for more than {threshold:.2f}s. Stopping...")
+        return True
+
     @staticmethod
     def duration_to_sequence(duration: float, sequences: List[Sequence]) -> int:
         d = 0.0
@@ -404,6 +432,10 @@ class HLSStreamWorker(SegmentedStreamWorker):
         return default
 
     def iter_segments(self):
+        self.playlist_reload_last \
+            = self.playlist_sequences_last \
+            = now()
+
         try:
             self.reload_playlist()
         except StreamError as err:
@@ -434,22 +466,43 @@ class HLSStreamWorker(SegmentedStreamWorker):
 
         total_duration = 0
         while not self.closed:
+            queued = False
             for sequence in filter(self.valid_sequence, self.playlist_sequences):
                 log.debug(f"Adding segment {sequence.num} to queue")
                 yield sequence
+                queued = True
+
                 total_duration += sequence.segment.duration
                 if self.duration_limit and total_duration >= self.duration_limit:
                     log.info(f"Stopping stream early after {self.duration_limit}")
                     return
 
                 # End of stream
-                stream_end = self.playlist_end and sequence.num >= self.playlist_end
+                stream_end = self.playlist_end is not None and sequence.num >= self.playlist_end
                 if self.closed or stream_end:
                     return
 
                 self.playlist_sequence = sequence.num + 1
 
-            if self.wait(self.playlist_reload_time):
+            if queued:
+                self.playlist_sequences_last = now()
+            elif self._segment_queue_timing_threshold_reached():
+                return
+
+            # Exclude playlist fetch+processing time from the overall playlist reload time
+            # and reload playlist in a strict time interval
+            time_completed = now()
+            time_elapsed = max(0.0, (time_completed - self.playlist_reload_last).total_seconds())
+            time_wait = max(0.0, self.playlist_reload_time - time_elapsed)
+            if self.wait(time_wait):
+                if time_wait > 0:
+                    # If we had to wait, then don't call now() twice and instead reference the timestamp from before
+                    # the wait() call, to prevent a shifting time offset due to the execution time.
+                    self.playlist_reload_last = time_completed + timedelta(seconds=time_wait)
+                else:
+                    # Otherwise, get the current time, as the reload interval already has shifted.
+                    self.playlist_reload_last = now()
+
                 try:
                     self.reload_playlist()
                 except StreamError as err:
@@ -476,7 +529,7 @@ class HLSStreamReader(FilteredStream, SegmentedStreamReader):
         super().__init__(stream)
 
 
-class MuxedHLSStream(MuxedStream):
+class MuxedHLSStream(MuxedStream["HLSStream"]):
     """
     Muxes multiple HLS video and audio streams into one output stream.
     """
@@ -485,24 +538,24 @@ class MuxedHLSStream(MuxedStream):
 
     def __init__(
         self,
-        session,
+        session: Streamlink,
         video: str,
         audio: Union[str, List[str]],
         url_master: Optional[str] = None,
         multivariant: Optional[M3U8] = None,
         force_restart: bool = False,
         ffmpeg_options: Optional[Dict[str, Any]] = None,
-        **args,
+        **kwargs,
     ):
         """
-        :param streamlink.Streamlink session: Streamlink session instance
+        :param session: Streamlink session instance
         :param video: Video stream URL
         :param audio: Audio stream URL or list of URLs
         :param url_master: The URL of the HLS playlist's multivariant playlist (deprecated)
         :param multivariant: The parsed multivariant playlist
         :param force_restart: Start from the beginning after reaching the playlist's end
         :param ffmpeg_options: Additional keyword arguments passed to :class:`ffmpegmux.FFMPEGMuxer`
-        :param args: Additional keyword arguments passed to :class:`HLSStream`
+        :param kwargs: Additional keyword arguments passed to :class:`HLSStream`
         """
 
         tracks = [video]
@@ -513,7 +566,7 @@ class MuxedHLSStream(MuxedStream):
             else:
                 tracks.append(audio)
         maps.extend(f"{i}:a" for i in range(1, len(tracks)))
-        substreams = [HLSStream(session, url, force_restart=force_restart, **args) for url in tracks]
+        substreams = [HLSStream(session, url, force_restart=force_restart, **kwargs) for url in tracks]
         ffmpeg_options = ffmpeg_options or {}
 
         super().__init__(session, *substreams, format="mpegts", maps=maps, **ffmpeg_options)
@@ -544,27 +597,27 @@ class HLSStream(HTTPStream):
 
     def __init__(
         self,
-        session_,
+        session: Streamlink,
         url: str,
         url_master: Optional[str] = None,
         multivariant: Optional[M3U8] = None,
         force_restart: bool = False,
         start_offset: float = 0,
         duration: Optional[float] = None,
-        **args,
+        **kwargs,
     ):
         """
-        :param streamlink.Streamlink session_: Streamlink session instance
+        :param session: Streamlink session instance
         :param url: The URL of the HLS playlist
         :param url_master: The URL of the HLS playlist's multivariant playlist (deprecated)
         :param multivariant: The parsed multivariant playlist
         :param force_restart: Start from the beginning after reaching the playlist's end
         :param start_offset: Number of seconds to be skipped from the beginning
         :param duration: Number of seconds until ending the stream
-        :param args: Additional keyword arguments passed to :meth:`requests.Session.request`
+        :param kwargs: Additional keyword arguments passed to :meth:`requests.Session.request`
         """
 
-        super().__init__(session_, url, **args)
+        super().__init__(session, url, **kwargs)
         self._url_master = url_master
         self.multivariant = multivariant if multivariant and multivariant.is_master else None
         self.force_restart = force_restart
@@ -607,8 +660,8 @@ class HLSStream(HTTPStream):
         return reader
 
     @classmethod
-    def _fetch_variant_playlist(cls, session, url: str, **request_params) -> Response:
-        res = session.http.get(url, exception=OSError, **request_params)
+    def _fetch_variant_playlist(cls, session, url: str, **request_args) -> Response:
+        res = session.http.get(url, exception=OSError, **request_args)
         res.encoding = "utf-8"
 
         return res
@@ -621,7 +674,7 @@ class HLSStream(HTTPStream):
     @classmethod
     def parse_variant_playlist(
         cls,
-        session_,
+        session: Streamlink,
         url: str,
         name_key: str = "name",
         name_prefix: str = "",
@@ -630,12 +683,12 @@ class HLSStream(HTTPStream):
         name_fmt: Optional[str] = None,
         start_offset: float = 0,
         duration: Optional[float] = None,
-        **request_params,
+        **kwargs,
     ) -> Dict[str, Union["HLSStream", "MuxedHLSStream"]]:
         """
         Parse a variant playlist and return its streams.
 
-        :param streamlink.Streamlink session_: Streamlink session instance
+        :param session: Streamlink session instance
         :param url: The URL of the variant playlist
         :param name_key: Prefer to use this key as stream name, valid keys are: name, pixels, bitrate
         :param name_prefix: Add this prefix to the stream names
@@ -644,19 +697,20 @@ class HLSStream(HTTPStream):
         :param name_fmt: A format string for the name, allowed format keys are: name, pixels, bitrate
         :param start_offset: Number of seconds to be skipped from the beginning
         :param duration: Number of second until ending the stream
-        :param request_params: Additional keyword arguments passed to :class:`HLSStream`, :class:`MuxedHLSStream`,
-                               or :py:meth:`requests.Session.request`
+        :param kwargs: Additional keyword arguments passed to :class:`HLSStream`, :class:`MuxedHLSStream`,
+                       or :py:meth:`requests.Session.request`
         """
 
-        locale = session_.localization
-        audio_select = session_.options.get("hls-audio-select")
+        locale = session.localization
+        audio_select = session.options.get("hls-audio-select")
 
-        res = cls._fetch_variant_playlist(session_, url, **request_params)
+        request_args = session.http.valid_request_args(**kwargs)
+        res = cls._fetch_variant_playlist(session, url, **request_args)
 
         try:
             multivariant = cls._get_variant_playlist(res)
         except ValueError as err:
-            raise OSError(f"Failed to parse playlist: {err}")
+            raise OSError(f"Failed to parse playlist: {err}") from err
 
         stream_name: Optional[str]
         stream: Union["HLSStream", "MuxedHLSStream"]
@@ -746,7 +800,7 @@ class HLSStream(HTTPStream):
             if check_streams:
                 # noinspection PyBroadException
                 try:
-                    session_.http.get(playlist.uri, **request_params)
+                    session.http.get(playlist.uri, **request_args)
                 except KeyboardInterrupt:
                     raise
                 except Exception:
@@ -754,7 +808,7 @@ class HLSStream(HTTPStream):
 
             external_audio = preferred_audio or default_audio or fallback_audio
 
-            if external_audio and FFMPEGMuxer.is_usable(session_):
+            if external_audio and FFMPEGMuxer.is_usable(session):
                 external_audio_msg = ", ".join([
                     f"(language={x.language}, name={x.name or 'N/A'})"
                     for x in external_audio
@@ -762,24 +816,24 @@ class HLSStream(HTTPStream):
                 log.debug(f"Using external audio tracks for stream {stream_name} {external_audio_msg}")
 
                 stream = MuxedHLSStream(
-                    session_,
+                    session,
                     video=playlist.uri,
                     audio=[x.uri for x in external_audio if x.uri],
                     multivariant=multivariant,
                     force_restart=force_restart,
                     start_offset=start_offset,
                     duration=duration,
-                    **request_params,
+                    **kwargs,
                 )
             else:
                 stream = cls(
-                    session_,
+                    session,
                     playlist.uri,
                     multivariant=multivariant,
                     force_restart=force_restart,
                     start_offset=start_offset,
                     duration=duration,
-                    **request_params,
+                    **kwargs,
                 )
 
             streams[stream_name] = stream
